/*
 * Copyright (C) 2007 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

#if !defined HATP_PARTIAL_PLAN_APPRAISER_HH_
#define HATP_PARTIAL_PLAN_APPRAISER_HH_

#include<string>
#include<vector>
#include<map>

class TaskInstance;
class TaskList;
class PlanTree;
class LightWorldBase;
class SocialRuleBase;

class PartialPlanAppraiser
{
private:
	/**
	* current projection cost
	*/
	float current_projection_cost;

	/**
	* current penalties
	*/
	float current_penalties;

	/**
	* bad decompo task names to scrutinize to detect badDecompositions
	*/
	std::map<std::string, std::vector<std::string> > badDecompoTaskNames;

	/**
	* bad decompo task names detection status (arranged in same order as badDecompoTaskNames)
	*/
	std::map<std::string, std::vector<bool> > badDecompoTaskNamesDetected;

	/**
	* bad decompo names associated to the current value of the penalty
	*/
	std::map<std::string,float> badDecompoPenalties;

	/**
	* undesirable sequence task names to scrutinize to detect badDecompositions
	*/
	std::map<std::string, std::vector<std::string> > undSeqTaskNames;

	/**
	* undesirable sequence task names detection status (arranged in same order as badDecompoTaskNames)
	*/
	std::map<std::string, std::vector<bool> > undSeqTaskNamesDetected;

	/**
	* undesirable sequence names associated to the current value of the penalty
	*/
	std::map<std::string,float> undSeqPenalties;

	/**
	* penalty associated to intricacy control
	*/
	float intricacyControlPenalty;

public:
	explicit PartialPlanAppraiser();
	PartialPlanAppraiser(PartialPlanAppraiser const& ppa);
	void addBadDecomposition(std::string const& name, std::vector<std::string> const& badDecompoSubtasks);
	void addUndesirableSequence(std::string const& name, std::vector<std::string> const& undSeqTasks);
	float getCurrentProjectionCost() const;
	float getCurrentPenalties() const;
	std::map<std::string, std::vector<std::string> > const& getBadDecompoTaskNames() const;
	std::map<std::string, std::vector<std::string> > const& getUndSeqTaskNames() const;
	std::map<std::string, std::vector<bool> > const& getBadDecompoTaskNamesDetected() const;
	std::map<std::string, std::vector<bool> > const& getUndSeqTaskNamesDetected() const;
	std::map<std::string,float> const& getBadDecompoPenalties() const;
	std::map<std::string,float> const& getUndSeqPenalties() const;
	unsigned int getIntricacyControlNbLinks() const;
	float getIntricacyControlPenalty() const;
	void addToCurrentProjectionCost(float toAdd);
	void updateBadDecompoFields(SocialRuleBase const& srb, LightWorldBase const& wb, PlanTree const& tree);
	void updateBadDecompoFields(TaskList const& TL, SocialRuleBase const& srb, LightWorldBase const& wb, PlanTree const& tree);
	void updateUndSeqFields(TaskInstance const& ti, SocialRuleBase const& srb, LightWorldBase const& wb, TaskList const& proj);
	void updateIntricacyControlFields(SocialRuleBase const& srb, TaskList const& proj);
	void init();
	PartialPlanAppraiser& operator=(PartialPlanAppraiser const& other);
	~PartialPlanAppraiser();
};

/**
 * constructor: set current_projection_cost to 0
 */
inline PartialPlanAppraiser::PartialPlanAppraiser()
{
	current_projection_cost=0.;
	current_penalties=0.;
	intricacyControlPenalty=0.;
}

/**
 * constructor by copying
 */
inline PartialPlanAppraiser::PartialPlanAppraiser(PartialPlanAppraiser const& ppa)
{
	current_projection_cost = ppa.getCurrentProjectionCost();
	current_penalties = ppa.getCurrentPenalties();
	badDecompoTaskNames = ppa.getBadDecompoTaskNames();
	undSeqTaskNames = ppa.getUndSeqTaskNames();
	badDecompoTaskNamesDetected = ppa.getBadDecompoTaskNamesDetected();
	undSeqTaskNamesDetected = ppa.getUndSeqTaskNamesDetected();
	badDecompoPenalties = ppa.getBadDecompoPenalties();
	undSeqPenalties = ppa.getUndSeqPenalties();
	intricacyControlPenalty = ppa.getIntricacyControlPenalty();
}

/**
 * return current_projection_cost
 */
inline float PartialPlanAppraiser::getCurrentProjectionCost() const
{
	return current_projection_cost;
}

/**
 * return current_score
 */
inline float PartialPlanAppraiser::getCurrentPenalties() const
{
	return current_penalties;
}

/**
 * return badDecompoTaskNames
 */
inline std::map<std::string, std::vector<std::string> > const& PartialPlanAppraiser::getBadDecompoTaskNames() const
{
	return badDecompoTaskNames;
}

/**
 * return undSeqTaskNames
 */
inline std::map<std::string, std::vector<std::string> > const& PartialPlanAppraiser::getUndSeqTaskNames() const
{
	return undSeqTaskNames;
}

/**
 * return badDecompoTaskNamesDetected
 */
inline std::map<std::string, std::vector<bool> > const& PartialPlanAppraiser::getBadDecompoTaskNamesDetected() const
{
	return badDecompoTaskNamesDetected;
}

/**
 * return undSeqTaskNamesDetected
 */
inline std::map<std::string, std::vector<bool> > const& PartialPlanAppraiser::getUndSeqTaskNamesDetected() const
{
	return undSeqTaskNamesDetected;
}

/**
 * return badDecompoPenalties
 */
inline std::map<std::string,float> const& PartialPlanAppraiser::getBadDecompoPenalties() const
{
	return badDecompoPenalties;
}

/**
 * return undSeqPenalties
 */
inline std::map<std::string,float> const& PartialPlanAppraiser::getUndSeqPenalties() const
{
	return undSeqPenalties;
}

/**
 * return intricacyControlPenalty
 */
inline float PartialPlanAppraiser::getIntricacyControlPenalty() const
{
	return intricacyControlPenalty;
}

/**
 * add toAdd to current_projection_cost
 */
inline void PartialPlanAppraiser::addToCurrentProjectionCost(float toAdd)
{
	current_projection_cost += toAdd;
}

/**
 * destructor: empty
 */
inline PartialPlanAppraiser::~PartialPlanAppraiser()
{}

#endif
