#include<iostream>

#include "Task.hh"
#include "TaskList.hh"
#include "WorldBase.hh"
#include "LightWorldBase.hh"
#include "WorldBaseReport.hh"
#include "HTNBase.hh"

using namespace std;

/** Constructor.
 * The variable Task::nbArgs is setted to the size of svTypes.
 * @param sName The task's name
 * @param svTypes The arguments' type for the task's call
 * @param bOperator WHether the task is an action (true) or a method (false)
 * @param preconditionsFunction The precondition function pointer
 * @param preconditionsWithReportFunction The precondition function pointer (with report)
 * @param emptyFunction The empty function pointer
 * @param decompositionsFunction The decomposition function pointer
 * @param effectsFunction The effect function pointer
 * @param effectsWithReportFunction The effect function pointer (with report)
 * @param costFunction The cost function pointer
 * @param timeFunction The duration function pointer
 */
Task::Task(std::string const& sName, std::vector<std::string> const& svTypes, bool bOperator,
	precondWithReportFunc preconditionsWithReportFunction, emptyFunc emptyFunction,
	decompoFunc decompositionsFunction, effectsFunc effectsFunction,
	effectsWithReportFunc effectsWithReportFunction, costFunc costFunction, timeFunc timeFunction)
	: name(sName), nbArgs(svTypes.size()), argTypes(svTypes), isOperator(bOperator),
	checkPrecondsWithReport(bOperator?preconditionsWithReportFunction:NULL),
	checkEmpty(bOperator?NULL:emptyFunction), getDecompos(bOperator?NULL:decompositionsFunction),
	applyEffects(bOperator?effectsFunction:NULL),
	applyEffectsWithReport(bOperator?effectsWithReportFunction:NULL),
	computeCost(bOperator?costFunction:NULL), computeTimeLimit(bOperator?timeFunction:NULL),
	communicationTask(false) {
	// we explore argTypes from the start and count the number of Agents before all other kinds of Entity
	nbInvolvedAgents=0;
	for(size_t i=0 ; i<nbArgs ; ++i) {
		if(argTypes[i] == AGENT) nbInvolvedAgents++;
		else break;
	}
}

/** Compare a given set of arguments' type with the task's set. Also check that the entities exists in the LightWorldBase with the right type.
 * @param args The set argument to check
 * @param wb The LightWorldBase
 * @return True if the set match the task's set and that every entity exist
 */
bool Task::checkTypes(std::vector<std::string> const& args, LightWorldBase const& wb) const {
	if(args.size()==nbArgs) {
		for(size_t i=0 ; i<args.size() ; ++i)
			if( !(wb.containEntity(args[i])) || wb.find(args[i])->getType() != argTypes[i] )
				if(argTypes[i] != "Entity" && argTypes[i] != "Attribute") return false;
		return true;
	}
	return false;
}

/** Compare a given set of arguments' type with the task's set. Also check that the entities exists in the WorldBase with the right type.
 * @param args The set argument to check
 * @param wb The WorldBase
 * @return True if the set match the task's set and that every entity exist
 */
bool Task::checkTypes(std::vector<std::string> const& args, WorldBase const& wb) const {
	if(args.size()==nbArgs) {
		for(size_t i=0 ; i<args.size() ; ++i)
			if( !(wb.containEntity(args[i])) || wb.find(args[i])->getType() != argTypes[i])
				return false;
		return true;
	}
	return false;
}

/** Compare a given set of arguments' type with the task's set. Also check that the entities exists in the given map with the right type.
 * This function is useful for declaring subTaskList/PlanTree especially for social rules as undesirable sequence and bad decomposition.
 * @param args The set argument to check
 * @param varAndTypes Map of types and name to match the set against
 * @return True if the set match the task's set and that every entity exist
 */
bool Task::checkTypes(std::vector<std::string> const& args, std::map<std::string, std::string> const& varAndTypes) const {
	if(args.size()==nbArgs) {
		map<string, string>::const_iterator it;
		for(size_t i=0 ; i<args.size() ; ++i) {
			it = varAndTypes.find(args[i]);
			if( it == varAndTypes.end() || it->second != argTypes[i])
				return false;
		}
		return true;
	}
	return false;
}

/** Change the type of task.
 * @param f New type, if f is trus then the task becomes a communication task
 */
void Task::setCommunicationTask(bool f) {
	this->communicationTask = f;
}

/** Create a string representing the Task.
 * @return String representing the Task
 */
string Task::toString() const {
	string s ;
	if(isOperator) s = "action - ";
	else s = "task - ";
	s += name + "(";
	for(size_t i=0 ; i<argTypes.size() ; ++i) {
		s += argTypes[i];
		if(i<argTypes.size()-1)
			s += ", ";
	}
	s += ")";
	return s;
}
